Package: lefser
Type: Package
Title: R implementation of the LEfSE method for microbiome biomarker
        discovery
Description: lefser is the R implementation of the popular microbiome 
  biomarker discovery too, LEfSe. It uses the Kruskal-Wallis test, 
  Wilcoxon-Rank Sum test, and Linear Discriminant Analysis to find biomarkers 
  from two-level classes (and optional sub-classes).
Version: 1.20.0
Date: 2025-04-22
Authors@R: c(
  person("Sehyun", "Oh", , "shbrief@gmail.com", c("cre", "ctb"), c(ORCID = "0000-0002-9490-3061")),
  person("Asya", "Khleborodova", , "asya.bioconductor@gmail.com", "aut"),
  person("Samuel", "Gamboa-Tuz", , "Samuel.Gamboa.Tuz@gmail.com", "ctb"),
  person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu", "ctb", c(ORCID = "0000-0002-3242-0582")),
  person("Ludwig", "Geistlinger", , "Ludwig.Geistlinger@sph.cuny.edu", "ctb", c(ORCID = "0000-0002-2495-5464")),
  person("Levi", "Waldron", , "levi.waldron@sph.cuny.edu", "ctb", c(ORCID = "0000-0003-2725-0694"))
  )
License: Artistic-2.0
Depends: SummarizedExperiment, R (>= 4.5.0)
Imports: coin, MASS, ggplot2, S4Vectors, stats, methods, utils, dplyr,
        testthat, tibble, tidyr, forcats, stringr, ggtree,
        BiocGenerics, ape, ggrepel, mia, purrr, tidyselect, treeio
Suggests: knitr, rmarkdown, curatedMetagenomicData, BiocStyle,
        phyloseq, pkgdown, covr, withr
Encoding: UTF-8
BugReports: https://github.com/waldronlab/lefser/issues
URL: https://github.com/waldronlab/lefser
VignetteBuilder: knitr
biocViews: Software, Sequencing, DifferentialExpression, Microbiome,
        StatisticalMethod, Classification
RoxygenNote: 7.3.2
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/lefser
git_branch: RELEASE_3_22
git_last_commit: c184c36
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 22:22:42 UTC; biocbuild
Author: Sehyun Oh [cre, ctb] (ORCID: <https://orcid.org/0000-0002-9490-3061>),
  Asya Khleborodova [aut],
  Samuel Gamboa-Tuz [ctb],
  Marcel Ramos [ctb] (ORCID: <https://orcid.org/0000-0002-3242-0582>),
  Ludwig Geistlinger [ctb] (ORCID:
    <https://orcid.org/0000-0002-2495-5464>),
  Levi Waldron [ctb] (ORCID: <https://orcid.org/0000-0003-2725-0694>)
Maintainer: Sehyun Oh <shbrief@gmail.com>
