# Generated by roxygen2: do not edit by hand

export(get_terminal_nodes)
export(lefser)
export(lefserClades)
export(lefserPlot)
export(lefserPlotClad)
export(lefserPlotFeat)
export(relativeAb)
export(rowNames2RowData)
import(SummarizedExperiment)
import(ggplot2)
importFrom(MASS,lda)
importFrom(coin,pvalue)
importFrom(coin,statistic)
importFrom(coin,wilcox_test)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,mutate)
importFrom(dplyr,row_number)
importFrom(ggtree,"%<+%")
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,kruskal.test)
importFrom(stats,reorder)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(testthat,capture_warnings)
importFrom(utils,head)
importFrom(utils,tail)
