#' Example dataset for lefser
#'
#' The ZellerG_2014 dataset contains microbiome count data for CRC patients and
#' controls. It was for curatedMetagenomicData using the script in the package
#' directory "data-raw".
#'
#' @usage data("zeller14")
#' @format A [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class]
#' with 1585 features, 199 samples
#' \describe{
#'   \item{study_condition}{adenoma, control, CRC}
#'   \item{age_category}{adult, senior}
#' }
#' @source \url{https://pubmed.ncbi.nlm.nih.gov/25432777/}
"zeller14"
